import pytest


@pytest.fixture(scope="module")
def property_max():
    return {
        "@type": "woql:And",
        "woql:query_list": [
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "owl:DatatypeProperty",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:range",
                    },
                    "woql:object": {"@type": "woql:Node", "woql:node": "xsd:string"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 2},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:domain",
                    },
                    "woql:object": {"@type": "woql:Node", "woql:node": "scm:A"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 3},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P_max"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "owl:Restriction",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 4},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P_max"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "owl:onProperty",
                    },
                    "woql:object": {"@type": "woql:Node", "woql:node": "scm:P"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 5},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P_max"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "owl:maxCardinality",
                    },
                    "woql:object": {
                        "@type": "woql:Datatype",
                        "woql:datatype": {
                            "@value": 4,
                            "@type": "xsd:nonNegativeInteger",
                        },
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 6},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:A"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:subClassOf",
                    },
                    "woql:object": {"@type": "woql:Node", "woql:node": "scm:P_max"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
        ],
    }


@pytest.fixture(scope="module")
def property_min():
    return {
        "@type": "woql:And",
        "woql:query_list": [
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "owl:DatatypeProperty",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:range",
                    },
                    "woql:object": {"@type": "woql:Node", "woql:node": "xsd:string"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 2},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:domain",
                    },
                    "woql:object": {"@type": "woql:Node", "woql:node": "scm:A"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 3},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P_min"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "owl:Restriction",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 4},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P_min"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "owl:onProperty",
                    },
                    "woql:object": {"@type": "woql:Node", "woql:node": "scm:P"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 5},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P_min"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "owl:minCardinality",
                    },
                    "woql:object": {
                        "@type": "woql:Datatype",
                        "woql:datatype": {
                            "@value": 2,
                            "@type": "xsd:nonNegativeInteger",
                        },
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 6},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:A"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:subClassOf",
                    },
                    "woql:object": {"@type": "woql:Node", "woql:node": "scm:P_min"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
        ],
    }


@pytest.fixture(scope="module")
def property_cardinalty():
    return {
        "@type": "woql:And",
        "woql:query_list": [
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "owl:DatatypeProperty",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:range",
                    },
                    "woql:object": {"@type": "woql:Node", "woql:node": "xsd:string"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 2},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:P"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:domain",
                    },
                    "woql:object": {"@type": "woql:Node", "woql:node": "scm:A"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 3},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {
                        "@type": "woql:Node",
                        "woql:node": "scm:P_cardinality",
                    },
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "owl:Restriction",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 4},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {
                        "@type": "woql:Node",
                        "woql:node": "scm:P_cardinality",
                    },
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "owl:onProperty",
                    },
                    "woql:object": {"@type": "woql:Node", "woql:node": "scm:P"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 5},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {
                        "@type": "woql:Node",
                        "woql:node": "scm:P_cardinality",
                    },
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "owl:cardinality",
                    },
                    "woql:object": {
                        "@type": "woql:Datatype",
                        "woql:datatype": {
                            "@value": 3,
                            "@type": "xsd:nonNegativeInteger",
                        },
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 6},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:A"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:subClassOf",
                    },
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "scm:P_cardinality",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
        ],
    }


@pytest.fixture(scope="module")
def chain_insert():
    return {
        "@type": "woql:And",
        "woql:query_list": [
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:query": {
                    "@type": "woql:AddTriple",
                    "woql:subject": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Node_ID",
                            "@type": "xsd:string",
                        },
                    },
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {"@value": "Type", "@type": "xsd:string"},
                    },
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:query": {
                    "@type": "woql:AddTriple",
                    "woql:subject": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Node_ID",
                            "@type": "xsd:string",
                        },
                    },
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:label"},
                    "woql:object": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Label",
                            "@type": "xsd:string",
                        },
                    },
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 2},
                "woql:query": {
                    "@type": "woql:AddTriple",
                    "woql:subject": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Node_ID",
                            "@type": "xsd:string",
                        },
                    },
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:comment",
                    },
                    "woql:object": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Description",
                            "@type": "xsd:string",
                        },
                    },
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 3},
                "woql:query": {
                    "@type": "woql:AddTriple",
                    "woql:subject": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Node_ID",
                            "@type": "xsd:string",
                        },
                    },
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:prop"},
                    "woql:object": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {"@value": "Prop", "@type": "xsd:string"},
                    },
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 4},
                "woql:query": {
                    "@type": "woql:AddTriple",
                    "woql:subject": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Node_ID",
                            "@type": "xsd:string",
                        },
                    },
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:prop"},
                    "woql:object": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Prop2",
                            "@type": "xsd:string",
                        },
                    },
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 5},
                "woql:query": {
                    "@type": "woql:AddTriple",
                    "woql:subject": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Node_ID",
                            "@type": "xsd:string",
                        },
                    },
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:subClassOf",
                    },
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "scm:myParentClass",
                    },
                },
            },
        ],
    }


@pytest.fixture(scope="module")
def chain_doctype():
    return {
        "@type": "woql:And",
        "woql:query_list": [
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:MyDoc"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {"@type": "woql:Node", "woql:node": "owl:Class"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:MyDoc"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:subClassOf",
                    },
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "terminus:Document",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 2},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:MyDoc"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:label"},
                    "woql:object": {
                        "@type": "woql:Datatype",
                        "woql:datatype": {
                            "@value": "abc",
                            "@type": "xsd:string",
                            "@language": "en",
                        },
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 3},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:MyDoc"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:comment",
                    },
                    "woql:object": {
                        "@type": "woql:Datatype",
                        "woql:datatype": {
                            "@value": "abcd",
                            "@type": "xsd:string",
                            "@language": "en",
                        },
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 4},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:prop"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "owl:DatatypeProperty",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 5},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:prop"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:range"},
                    "woql:object": {"@type": "woql:Node", "woql:node": "xsd:dateTime"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 6},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:prop"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:domain",
                    },
                    "woql:object": {"@type": "woql:Node", "woql:node": "scm:MyDoc"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 7},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:prop"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:label"},
                    "woql:object": {
                        "@type": "woql:Datatype",
                        "woql:datatype": {
                            "@value": "aaa",
                            "@type": "xsd:string",
                            "@language": "en",
                        },
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 8},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:prop2"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "owl:DatatypeProperty",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 9},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:prop2"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:range"},
                    "woql:object": {"@type": "woql:Node", "woql:node": "xsd:integer"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 10},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:prop2"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:domain",
                    },
                    "woql:object": {"@type": "woql:Node", "woql:node": "scm:MyDoc"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 11},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:prop2"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:label"},
                    "woql:object": {
                        "@type": "woql:Datatype",
                        "woql:datatype": {
                            "@value": "abe",
                            "@type": "xsd:string",
                            "@language": "en",
                        },
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
        ],
    }



WoqlLimitStart = {
    "@type": "woql:Limit",
    "woql:limit": {
        "@type": "woql:Datatype",
        "woql:datatype": {"@type": "xsd:nonNegativeInteger", "@value": 10},
    },
    "woql:query": {
        "@type": "woql:Triple",
        "woql:subject": {
            "@type": "woql:Variable",
            "woql:variable_name": {"@value": "Subject", "@type": "xsd:string"},
        },
        "woql:predicate": {
            "@type": "woql:Variable",
            "woql:variable_name": {"@value": "Predicate", "@type": "xsd:string"},
        },
        "woql:object": {
            "@type": "woql:Variable",
            "woql:variable_name": {"@value": "Object", "@type": "xsd:string"},
        },
    },
}



import pytest


@pytest.fixture(scope="module")
def triple_opt():
    return {
        "@type": "woql:Optional",
        "woql:query": {
            "@type": "woql:Triple",
            "woql:subject": {"@type": "woql:Node", "woql:node": "doc:a"},
            "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:b"},
            "woql:object": {
                "@type": "woql:Datatype",
                "woql:datatype": {"@type": "xsd:string", "@value": "c"},
            },
        },
    }


@pytest.fixture(scope="module")
def quad_opt():
    return {
        "@type": "woql:Optional",
        "woql:query": {
            "@type": "woql:Quad",
            "woql:subject": {"@type": "woql:Node", "woql:node": "doc:a"},
            "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:b"},
            "woql:object": {
                "@type": "woql:Datatype",
                "woql:datatype": {"@type": "xsd:string", "@value": "c"},
            },
            "woql:graph_filter": {"@type": "xsd:string", "@value": "d"},
        },
    }



WoqlDoctype = {
    "@type": "woql:And",
    "woql:query_list": [
        {
            "@type": "woql:QueryListElement",
            "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
            "woql:query": {
                "@type": "woql:AddQuad",
                "woql:subject": {"@type": "woql:Node", "woql:node": "scm:Station"},
                "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                "woql:object": {"@type": "woql:Node", "woql:node": "owl:Class"},
                "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
            },
        },
        {
            "@type": "woql:QueryListElement",
            "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
            "woql:query": {
                "@type": "woql:AddQuad",
                "woql:subject": {"@type": "woql:Node", "woql:node": "scm:Station"},
                "woql:predicate": {
                    "@type": "woql:Node",
                    "woql:node": "rdfs:subClassOf",
                },
                "woql:object": {"@type": "woql:Node", "woql:node": "terminus:Document"},
                "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
            },
        },
    ],
}



server_records_from_cap = {
    "@id": "doc:server",
    "@type": "terminus:Server",
    "rdfs:comment": {"@language": "en", "@value": "The current Database Server itself"},
    "rdfs:label": {"@language": "en", "@value": "The DB server"},
    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
    "terminus:resource_includes": [
        {"@id": "doc:Database%5fadmin%7C5534534", "@type": "terminus:Database"},
        {"@id": "doc:Database%5fadmin%7Caaaaaa", "@type": "terminus:Database"},
        {"@id": "doc:Database%5fadmin%7Cadsasasddsa", "@type": "terminus:Database"},
        {"@id": "doc:Database%5fadmin%7Cblah", "@type": "terminus:Database"},
        {"@id": "doc:Database%5fadmin%7Cdaassd", "@type": "terminus:Database"},
        {"@id": "doc:Database%5fadmin%7Cdassadds", "@type": "terminus:Database"},
        {"@id": "doc:Database%5fadmin%7Cddd", "@type": "terminus:Database"},
        {"@id": "doc:Database%5fadmin%7Cffff", "@type": "terminus:Database"},
        {"@id": "doc:Database%5fadmin%7Cfffffff", "@type": "terminus:Database"},
        {"@id": "doc:Database%5fadmin%7Cggggggggg", "@type": "terminus:Database"},
        {"@id": "doc:Database%5fadmin%7Ctwretwert", "@type": "terminus:Database"},
        {"@id": "doc:terminus", "@type": "terminus:Database"},
    ],
    "terminus:resource_name": {
        "@type": "xsd:string",
        "@value": "http://localhost:6363",
    },
    "terminus:authority": [
        "terminus:branch",
        "terminus:class_frame",
        "terminus:clone",
        "terminus:commit_read_access",
        "terminus:commit_write_access",
        "terminus:create_database",
        "terminus:delete_database",
        "terminus:fetch",
        "terminus:inference_read_access",
        "terminus:inference_write_access",
        "terminus:instance_read_access",
        "terminus:instance_write_access",
        "terminus:manage",
        "terminus:meta_read_access",
        "terminus:meta_write_access",
        "terminus:push",
        "terminus:rebase",
        "terminus:schema_read_access",
        "terminus:schema_write_access",
    ],
}



# import sys
# sys.path.append('woqlclient')
import unittest.mock as mock

import pytest
import requests
from terminusdb_client.woqlclient.woqlClient import WOQLClient

from ..__version__ import __version__
from .mockResponse import mocked_requests
from .woqljson.woqlStarJson import WoqlStar


def mock_func_with_1arg(_):
    return True


def mock_func_with_2arg(first, second):
    return True


def mock_func_no_arg():
    return True


@mock.patch("requests.get", side_effect=mocked_requests)
def test_connection(mocked_requests, monkeypatch):

    woql_client = WOQLClient("http://localhost:6363")

    # before connect it connection is empty
    assert woql_client.conCapabilities.connection == {}

    woql_client.connect(key="root", account="admin", user="admin")

    requests.get.assert_called_once_with(
        "http://localhost:6363/", headers={"Authorization": "Basic YWRtaW46cm9vdA=="}
    )


@mock.patch("requests.post", side_effect=mocked_requests)
@mock.patch("requests.get", side_effect=mocked_requests)
def test_create_database(mocked_requests, mocked_requests2):
    woql_client = WOQLClient(
        "http://localhost:6363", user="admin", key="root", account="admin"
    )
    woql_client.connect()
    assert woql_client.basic_auth() == "admin:root"

    woql_client.create_database(
        "myFirstTerminusDB",
        "admin",
        label="my first db",
        description="my first db comment",
        include_schema=False,
    )

    requests.post.assert_called_once_with(
        "http://localhost:6363/db/admin/myFirstTerminusDB",
        headers={
            "Authorization": "Basic YWRtaW46cm9vdA==",
            "content-type": "application/json",
        },
        json={
            "label": "my first db",
            "comment": "my first db comment",
            "prefixes": {
                "scm": "terminusdb://admin/myFirstTerminusDB/schema#",
                "doc": "terminusdb://admin/myFirstTerminusDB/data/",
            },
        },
    )


@mock.patch("requests.post", side_effect=mocked_requests)
@mock.patch("requests.get", side_effect=mocked_requests)
@mock.patch("terminusdb_client.woqlclient.woqlClient.WOQLClient.create_graph")
def test_create_database_with_schema(
    mocked_requests, mocked_requests2, create_schema_obj
):
    woql_client = WOQLClient(
        "http://localhost:6363", user="admin", key="root", account="admin"
    )
    woql_client.connect()
    assert woql_client.basic_auth() == "admin:root"

    woql_client.create_database(
        "myFirstTerminusDB",
        "admin",
        label="my first db",
        description="my first db comment",
    )

    WOQLClient.create_graph.assert_called_once_with(
        "schema", "main", f"Python client {__version__} message: Creating schema graph"
    )


@mock.patch("requests.post", side_effect=mocked_requests)
@mock.patch("requests.get", side_effect=mocked_requests)
def test_create_database_and_change_account(mocked_requests, mocked_requests2):
    woql_client = WOQLClient(
        "http://localhost:6363", user="admin", key="root", account="admin"
    )
    woql_client.connect()
    woql_client.create_database(
        "myFirstTerminusDB",
        "my_new_account",
        label="my first db",
        description="my first db comment",
        include_schema=False,
    )

    requests.post.assert_called_once_with(
        "http://localhost:6363/db/my_new_account/myFirstTerminusDB",
        headers={
            "Authorization": "Basic YWRtaW46cm9vdA==",
            "content-type": "application/json",
        },
        json={
            "label": "my first db",
            "comment": "my first db comment",
            "prefixes": {
                "scm": "terminusdb://my_new_account/myFirstTerminusDB/schema#",
                "doc": "terminusdb://my_new_account/myFirstTerminusDB/data/",
            },
        },
    )

    assert woql_client.basic_auth() == "admin:root"


@mock.patch("requests.delete", side_effect=mocked_requests)
@mock.patch("requests.get", side_effect=mocked_requests)
def test_delete_database(mocked_requests, mocked_requests2, monkeypatch):
    woql_client = WOQLClient("http://localhost:6363")

    woql_client.connect(user="admin", account="admin", key="root")

    monkeypatch.setattr(
        woql_client.conCapabilities, "capabilities_permit", mock_func_with_1arg
    )

    monkeypatch.setattr(woql_client.conCapabilities, "remove_db", mock_func_with_2arg)

    woql_client.delete_database("myFirstTerminusDB")

    requests.delete.assert_called_once_with(
        "http://localhost:6363/db/admin/myFirstTerminusDB",
        headers={"Authorization": "Basic YWRtaW46cm9vdA=="},
    )


@mock.patch("requests.get", side_effect=mocked_requests)
@mock.patch("requests.post", side_effect=mocked_requests)
def test_branch(mocked_requests, mocked_requests2):
    woql_client = WOQLClient("http://localhost:6363")
    woql_client.connect(user="admin", account="admin", key="root", db="myDBName")
    woql_client.branch("my_new_branch")

    requests.post.assert_called_once_with(
        "http://localhost:6363/branch/admin/myDBName/local/branch/my_new_branch",
        headers={
            "Authorization": "Basic YWRtaW46cm9vdA==",
            "content-type": "application/json",
        },
        json={"origin": "admin/myDBName/local/branch/master"},
    )


@mock.patch("requests.get", side_effect=mocked_requests)
@mock.patch("requests.post", side_effect=mocked_requests)
def test_create_graph(mocked_requests, mocked_requests2):
    woql_client = WOQLClient("http://localhost:6363")
    woql_client.connect(user="admin", account="admin", key="root", db="myDBName")

    woql_client.create_graph("instance", "mygraph", "add a new graph")

    requests.post.assert_called_once_with(
        "http://localhost:6363/graph/admin/myDBName/local/branch/master/instance/mygraph",
        headers={
            "Authorization": "Basic YWRtaW46cm9vdA==",
            "content-type": "application/json",
        },
        json={
            "commit_info": {
                "author": "admin Server Admin User",
                "message": "add a new graph",
            }
        },
    )


@mock.patch("requests.get", side_effect=mocked_requests)
@mock.patch("requests.post", side_effect=mocked_requests)
def test_wrong_graph_type(mocked_requests, mocked_requests2):
    woql_client = WOQLClient("http://localhost:6363")
    woql_client.connect(user="admin", account="admin", key="root", db="myDBName")

    with pytest.raises(ValueError):
        woql_client.create_graph("wrong_graph_name", "mygraph", "add a new graph")


@mock.patch("requests.get", side_effect=mocked_requests)
@mock.patch("requests.delete", side_effect=mocked_requests)
def test_delete_graph(mocked_requests, mocked_requests2):
    woql_client = WOQLClient("http://localhost:6363")
    woql_client.connect(user="admin", account="admin", key="root", db="myDBName")

    woql_client.delete_graph("instance", "mygraph", "add a new graph")

    requests.delete.assert_called_once_with(
        "http://localhost:6363/graph/admin/myDBName/local/branch/master/instance/mygraph?author=admin+Server+Admin+User&message=add+a+new+graph",
        headers={"Authorization": "Basic YWRtaW46cm9vdA=="},
    )


@mock.patch("requests.get", side_effect=mocked_requests)
def test_get_triples(mocked_requests):
    woql_client = WOQLClient("http://localhost:6363")
    woql_client.connect(user="admin", account="admin", key="root", db="myDBName")

    woql_client.get_triples("instance", "mygraph")

    requests.get.assert_called_with(
        "http://localhost:6363/triples/admin/myDBName/local/branch/master/instance/mygraph",
        headers={"Authorization": "Basic YWRtaW46cm9vdA=="},
    )


@mock.patch("requests.post", side_effect=mocked_requests)
@mock.patch("requests.get", side_effect=mocked_requests)
def test_query(mocked_requests, mocked_requests2):
    woql_client = WOQLClient("http://localhost:6363")
    woql_client.connect(user="admin", account="admin", key="root", db="myDBName")

    # WoqlStar is the query in json-ld

    woql_client.query(WoqlStar)

    requests.post.assert_called_once_with(
        "http://localhost:6363/woql/admin/myDBName/local/branch/master",
        headers={
            "Authorization": "Basic YWRtaW46cm9vdA==",
            "content-type": "application/json",
        },
        json={"query": WoqlStar},
    )



import pytest


@pytest.fixture(scope="module")
def doctype_without():
    return {
        "@type": "woql:And",
        "woql:query_list": [
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                    "woql:object": {"@type": "woql:Node", "woql:node": "owl:Class"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:Station"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "terminus:Document",
                    },
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:subClassOf",
                    },
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:Station"},
                },
            },
        ],
    }


@pytest.fixture(scope="module")
def doctype_with_label():
    return {
        "@type": "woql:And",
        "woql:query_list": [
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:Station"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {"@type": "woql:Node", "woql:node": "owl:Class"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:Station"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:subClassOf",
                    },
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "terminus:Document",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 2},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:Station"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:label"},
                    "woql:object": {
                        "@type": "woql:Datatype",
                        "woql:datatype": {
                            "@value": "Station Object",
                            "@type": "xsd:string",
                            "@language": "en",
                        },
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
        ],
    }


@pytest.fixture(scope="module")
def doctype_with_des():
    return {
        "@type": "woql:And",
        "woql:query_list": [
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:Station"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {"@type": "woql:Node", "woql:node": "owl:Class"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:Station"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:subClassOf",
                    },
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "terminus:Document",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 2},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:Station"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:label"},
                    "woql:object": {
                        "@type": "woql:Datatype",
                        "woql:datatype": {
                            "@value": "Station Object",
                            "@type": "xsd:string",
                            "@language": "en",
                        },
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 3},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:Station"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:comment",
                    },
                    "woql:object": {
                        "@type": "woql:Datatype",
                        "woql:datatype": {
                            "@value": "A bike station object.",
                            "@type": "xsd:string",
                            "@language": "en",
                        },
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
        ],
    }



ConnectResponse = {
    "@context": {
        "doc": "terminus:///terminus/document/",
        "layer": "http://terminusdb.com/schema/layer#",
        "owl": "http://www.w3.org/2002/07/owl#",
        "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
        "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
        "ref": "http://terminusdb.com/schema/ref#",
        "repo": "http://terminusdb.com/schema/repository#",
        "terminus": "http://terminusdb.com/schema/terminus#",
        "vio": "http://terminusdb.com/schema/vio#",
        "woql": "http://terminusdb.com/schema/woql#",
        "xdd": "http://terminusdb.com/schema/xdd#",
        "xsd": "http://www.w3.org/2001/XMLSchema#",
    },
    "@id": "doc:admin",
    "@type": "terminus:User",
    "rdfs:comment": {
        "@language": "en",
        "@value": "This is the server super user account",
    },
    "rdfs:label": {"@language": "en", "@value": "Server Admin User"},
    "terminus:agent_key_hash": {
        "@type": "xsd:string",
        "@value": "$pbkdf2-sha512$t=32768$b7+2Xn5XkuluDASL34WSlg$eORRJ0QzUGa5ET5BRFOd06NmX7hIvU6cWp86zLdMB0Sgu6YMhR2XT3ayL1cDaBs+5EhOPqAKp2BitYvqIsVldA",
    },
    "terminus:agent_name": {"@type": "xsd:string", "@value": "admin"},
    "terminus:authority": {
        "@id": "doc:access_all_areas",
        "@type": "terminus:Capability",
        "rdfs:comment": {"@language": "en", "@value": "Access all server functions"},
        "rdfs:label": {"@language": "en", "@value": "All Capabilities"},
        "terminus:access": {
            "@id": "doc:server_access",
            "@type": "terminus:Access",
            "terminus:action": [
                {"@id": "terminus:branch", "@type": "terminus:DBAction"},
                {"@id": "terminus:class_frame", "@type": "terminus:DBAction"},
                {"@id": "terminus:clone", "@type": "terminus:DBAction"},
                {"@id": "terminus:commit_read_access", "@type": "terminus:DBAction"},
                {"@id": "terminus:commit_write_access", "@type": "terminus:DBAction"},
                {"@id": "terminus:create_database", "@type": "terminus:DBAction"},
                {"@id": "terminus:delete_database", "@type": "terminus:DBAction"},
                {"@id": "terminus:fetch", "@type": "terminus:DBAction"},
                {"@id": "terminus:inference_read_access", "@type": "terminus:DBAction"},
                {
                    "@id": "terminus:inference_write_access",
                    "@type": "terminus:DBAction",
                },
                {"@id": "terminus:instance_read_access", "@type": "terminus:DBAction"},
                {"@id": "terminus:instance_write_access", "@type": "terminus:DBAction"},
                {"@id": "terminus:manage", "@type": "terminus:DBAction"},
                {"@id": "terminus:meta_read_access", "@type": "terminus:DBAction"},
                {"@id": "terminus:meta_write_access", "@type": "terminus:DBAction"},
                {"@id": "terminus:push", "@type": "terminus:DBAction"},
                {"@id": "terminus:rebase", "@type": "terminus:DBAction"},
                {"@id": "terminus:schema_read_access", "@type": "terminus:DBAction"},
                {"@id": "terminus:schema_write_access", "@type": "terminus:DBAction"},
            ],
            "terminus:authority_scope": [
                {
                    "@id": "doc:Database%5fadmin%7C5534534",
                    "@type": "terminus:Database",
                    "rdfs:comment": {"@language": "en", "@value": "345345"},
                    "rdfs:label": [
                        {"@language": "en", "@value": "3453"},
                        {"@language": "en", "@value": "admin|5534534"},
                    ],
                    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
                    "terminus:database_state": {
                        "@id": "terminus:finalized",
                        "@type": "terminus:DatabaseState",
                    },
                    "terminus:resource_name": {
                        "@type": "xsd:string",
                        "@value": "admin|5534534",
                    },
                },
                {
                    "@id": "doc:Database%5fadmin%7Caaaaaa",
                    "@type": "terminus:Database",
                    "rdfs:comment": {"@language": "en", "@value": "dasdasdds"},
                    "rdfs:label": [
                        {"@language": "en", "@value": "admin|aaaaaa"},
                        {"@language": "en", "@value": "dadsds"},
                    ],
                    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
                    "terminus:database_state": {
                        "@id": "terminus:finalized",
                        "@type": "terminus:DatabaseState",
                    },
                    "terminus:resource_name": {
                        "@type": "xsd:string",
                        "@value": "admin|aaaaaa",
                    },
                },
                {
                    "@id": "doc:Database%5fadmin%7Cadsasasddsa",
                    "@type": "terminus:Database",
                    "rdfs:comment": {"@language": "en", "@value": "asdsadsda"},
                    "rdfs:label": [
                        {"@language": "en", "@value": "admin|adsasasddsa"},
                        {"@language": "en", "@value": "asdasdasd"},
                    ],
                    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
                    "terminus:database_state": {
                        "@id": "terminus:finalized",
                        "@type": "terminus:DatabaseState",
                    },
                    "terminus:resource_name": {
                        "@type": "xsd:string",
                        "@value": "admin|adsasasddsa",
                    },
                },
                {
                    "@id": "doc:Database%5fadmin%7Cblah",
                    "@type": "terminus:Database",
                    "rdfs:comment": {"@language": "en", "@value": "adfadf"},
                    "rdfs:label": [
                        {"@language": "en", "@value": "admin|blah"},
                        {"@language": "en", "@value": "better freaking work"},
                    ],
                    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
                    "terminus:database_state": {
                        "@id": "terminus:finalized",
                        "@type": "terminus:DatabaseState",
                    },
                    "terminus:resource_name": {
                        "@type": "xsd:string",
                        "@value": "admin|blah",
                    },
                },
                {
                    "@id": "doc:Database%5fadmin%7Cdaassd",
                    "@type": "terminus:Database",
                    "rdfs:comment": {"@language": "en", "@value": "asdasdsd"},
                    "rdfs:label": [
                        {"@language": "en", "@value": "admin|daassd"},
                        {"@language": "en", "@value": "asdsd"},
                    ],
                    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
                    "terminus:database_state": {
                        "@id": "terminus:finalized",
                        "@type": "terminus:DatabaseState",
                    },
                    "terminus:resource_name": {
                        "@type": "xsd:string",
                        "@value": "admin|daassd",
                    },
                },
                {
                    "@id": "doc:Database%5fadmin%7Cdassadds",
                    "@type": "terminus:Database",
                    "rdfs:comment": {"@language": "en", "@value": "asdsdsad"},
                    "rdfs:label": [
                        {"@language": "en", "@value": "admin|dassadds"},
                        {"@language": "en", "@value": "asdasdds"},
                    ],
                    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
                    "terminus:database_state": {
                        "@id": "terminus:finalized",
                        "@type": "terminus:DatabaseState",
                    },
                    "terminus:resource_name": {
                        "@type": "xsd:string",
                        "@value": "admin|dassadds",
                    },
                },
                {
                    "@id": "doc:Database%5fadmin%7Cddd",
                    "@type": "terminus:Database",
                    "rdfs:comment": {"@language": "en", "@value": "ddds"},
                    "rdfs:label": [
                        {"@language": "en", "@value": "admin|ddd"},
                        {"@language": "en", "@value": "ddd"},
                    ],
                    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
                    "terminus:database_state": {
                        "@id": "terminus:finalized",
                        "@type": "terminus:DatabaseState",
                    },
                    "terminus:resource_name": {
                        "@type": "xsd:string",
                        "@value": "admin|ddd",
                    },
                },
                {
                    "@id": "doc:Database%5fadmin%7Cffff",
                    "@type": "terminus:Database",
                    "rdfs:comment": {"@language": "en", "@value": "fffff"},
                    "rdfs:label": [
                        {"@language": "en", "@value": "admin|ffff"},
                        {"@language": "en", "@value": "fff"},
                    ],
                    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
                    "terminus:database_state": {
                        "@id": "terminus:finalized",
                        "@type": "terminus:DatabaseState",
                    },
                    "terminus:resource_name": {
                        "@type": "xsd:string",
                        "@value": "admin|ffff",
                    },
                },
                {
                    "@id": "doc:Database%5fadmin%7Cfffffff",
                    "@type": "terminus:Database",
                    "rdfs:comment": {"@language": "en", "@value": "fff"},
                    "rdfs:label": [
                        {"@language": "en", "@value": "admin|fffffff"},
                        {"@language": "en", "@value": "fffasdfasdf"},
                    ],
                    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
                    "terminus:database_state": {
                        "@id": "terminus:finalized",
                        "@type": "terminus:DatabaseState",
                    },
                    "terminus:resource_name": {
                        "@type": "xsd:string",
                        "@value": "admin|fffffff",
                    },
                },
                {
                    "@id": "doc:Database%5fadmin%7Cggggggggg",
                    "@type": "terminus:Database",
                    "rdfs:comment": {"@language": "en", "@value": "gg"},
                    "rdfs:label": [
                        {"@language": "en", "@value": "admin|ggggggggg"},
                        {"@language": "en", "@value": "gg"},
                    ],
                    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
                    "terminus:database_state": {
                        "@id": "terminus:finalized",
                        "@type": "terminus:DatabaseState",
                    },
                    "terminus:resource_name": {
                        "@type": "xsd:string",
                        "@value": "admin|ggggggggg",
                    },
                },
                {
                    "@id": "doc:Database%5fadmin%7Ctwretwert",
                    "@type": "terminus:Database",
                    "rdfs:comment": {"@language": "en", "@value": "adfadf"},
                    "rdfs:label": [
                        {"@language": "en", "@value": "admin|twretwert"},
                        {"@language": "en", "@value": "dgsdfgsg"},
                    ],
                    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
                    "terminus:database_state": {
                        "@id": "terminus:finalized",
                        "@type": "terminus:DatabaseState",
                    },
                    "terminus:resource_name": {
                        "@type": "xsd:string",
                        "@value": "admin|twretwert",
                    },
                },
                {
                    "@id": "doc:terminus",
                    "@type": "terminus:Database",
                    "rdfs:comment": {
                        "@language": "en",
                        "@value": "The master database contains the meta-data about databases, users and roles",
                    },
                    "rdfs:label": {"@language": "en", "@value": "Master Database"},
                    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
                    "terminus:resource_name": {
                        "@type": "xsd:string",
                        "@value": "terminus",
                    },
                },
                {
                    "@id": "doc:server",
                    "@type": "terminus:Server",
                    "rdfs:comment": {
                        "@language": "en",
                        "@value": "The current Database Server itself",
                    },
                    "rdfs:label": {"@language": "en", "@value": "The DB server"},
                    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
                    "terminus:resource_includes": [
                        {
                            "@id": "doc:Database%5fadmin%7C5534534",
                            "@type": "terminus:Database",
                        },
                        {
                            "@id": "doc:Database%5fadmin%7Caaaaaa",
                            "@type": "terminus:Database",
                        },
                        {
                            "@id": "doc:Database%5fadmin%7Cadsasasddsa",
                            "@type": "terminus:Database",
                        },
                        {
                            "@id": "doc:Database%5fadmin%7Cblah",
                            "@type": "terminus:Database",
                        },
                        {
                            "@id": "doc:Database%5fadmin%7Cdaassd",
                            "@type": "terminus:Database",
                        },
                        {
                            "@id": "doc:Database%5fadmin%7Cdassadds",
                            "@type": "terminus:Database",
                        },
                        {
                            "@id": "doc:Database%5fadmin%7Cddd",
                            "@type": "terminus:Database",
                        },
                        {
                            "@id": "doc:Database%5fadmin%7Cffff",
                            "@type": "terminus:Database",
                        },
                        {
                            "@id": "doc:Database%5fadmin%7Cfffffff",
                            "@type": "terminus:Database",
                        },
                        {
                            "@id": "doc:Database%5fadmin%7Cggggggggg",
                            "@type": "terminus:Database",
                        },
                        {
                            "@id": "doc:Database%5fadmin%7Ctwretwert",
                            "@type": "terminus:Database",
                        },
                        {"@id": "doc:terminus", "@type": "terminus:Database"},
                    ],
                    "terminus:resource_name": {
                        "@type": "xsd:string",
                        "@value": "http://localhost:6363",
                    },
                },
            ],
        },
    },
}



class Error(Exception):
    """Exception that is base class for all other error exceptions."""

    def __init__(self, msg=None, url=None, err_obj=None):
        super().__init__()
        self.msg = msg
        self.url = url
        self.errorObj = err_obj

    def __str__(self):
        return self.msg


class InterfaceError(Error):
    """Exception for errors related to the interface."""


class DatabaseError(Error):
    """Exception for errors related to the database."""


class AccessDeniedError(DatabaseError):
    pass


class APIError(DatabaseError):
    """Exceptions to do with return messages from HTTP"""

    def __init__(self, msg=None, url=None, err_obj=None, status_code=None):
        super().__init__()
        self.msg = msg
        self.url = url
        self.errorObj = err_obj
        self.status_code = status_code


class InvalidURIError(Error):
    pass



WOQL_WHEN_JSON = {
    "@type": "woql:When",
    "woql:query": {"@type": "woql:True"},
    "woql:consequent": {
        "@type": "woql:AddQuad",
        "woql:subject": {"@type": "woql:Node", "woql:node": "scm:id"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
        "woql:object": {"@type": "woql:Node", "woql:node": "owl:Class"},
        "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
    },
}



db_record_obj = {
    "@id": "doc:Database%5fadmin%7Caaaaaa",
    "@type": "terminus:Database",
    "rdfs:comment": {"@language": "en", "@value": "dasdasdds"},
    "rdfs:label": [
        {"@language": "en", "@value": "admin|aaaaaa"},
        {"@language": "en", "@value": "dadsds"},
    ],
    "terminus:allow_origin": {"@type": "xsd:string", "@value": "*"},
    "terminus:database_state": {
        "@id": "terminus:finalized",
        "@type": "terminus:DatabaseState",
    },
    "terminus:resource_name": {"@type": "xsd:string", "@value": "admin|aaaaaa"},
    "terminus:authority": [
        "terminus:branch",
        "terminus:class_frame",
        "terminus:clone",
        "terminus:commit_read_access",
        "terminus:commit_write_access",
        "terminus:create_database",
        "terminus:delete_database",
        "terminus:fetch",
        "terminus:inference_read_access",
        "terminus:inference_write_access",
        "terminus:instance_read_access",
        "terminus:instance_write_access",
        "terminus:manage",
        "terminus:meta_read_access",
        "terminus:meta_write_access",
        "terminus:push",
        "terminus:rebase",
        "terminus:schema_read_access",
        "terminus:schema_write_access",
    ],
}



WOQL_STAR = {
    "@type": "woql:Triple",
    "woql:object": {
        "@type": "woql:Variable",
        "woql:variable_name": {"@type": "xsd:string", "@value": "Object"},
    },
    "woql:predicate": {
        "@type": "woql:Variable",
        "woql:variable_name": {"@type": "xsd:string", "@value": "Predicate"},
    },
    "woql:subject": {
        "@type": "woql:Variable",
        "woql:variable_name": {"@type": "xsd:string", "@value": "Subject"},
    },
}

WOQL_JSON = {
    "quadJson": {
        "@type": "woql:Quad",
        "woql:subject": {"@type": "woql:Node", "woql:node": "doc:a"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:b"},
        "woql:object": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:string", "@value": "c"},
        },
        "woql:graph_filter": {"@type": "xsd:string", "@value": "d"},
    },
    "tripleJson": {
        "@type": "woql:Triple",
        "woql:subject": {"@type": "woql:Node", "woql:node": "doc:a"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:b"},
        "woql:object": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:string", "@value": "c"},
        },
    },
    "getJson": {
        "@type": "woql:Get",
        "woql:as_vars": [
            {
                "@type": "woql:NamedAsVar",
                "woql:identifier": {"@type": "xsd:string", "@value": "M"},
                "woql:variable_name": {"@type": "xsd:string", "@value": "a"},
                "woql:var_type": {"@type": "xsd:anyURI", "@value": "p"},
            }
        ],
        "woql:query_resource": "Target",
    },
    "memberJson": {
        "@type": "woql:Member",
        "woql:member": {
            "@type": "woql:Variable",
            "woql:variable_name": {"@value": "member", "@type": "xsd:string"},
        },
        "woql:member_list": {
            "@type": "woql:Variable",
            "woql:variable_name": {"@value": "list_obj", "@type": "xsd:string"},
        },
    },
    "groupbyJson": {
        "@type": "woql:GroupBy",
        "woql:group_by": [
            {
                "@type": "woql:VariableListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:variable_name": {"@value": "A", "@type": "xsd:string"},
            },
            {
                "@type": "woql:VariableListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:variable_name": {"@value": "B", "@type": "xsd:string"},
            },
        ],
        "woql:group_template": [
            {
                "@type": "woql:VariableListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:variable_name": {"@value": "C", "@type": "xsd:string"},
            }
        ],
        "woql:grouped": {
            "@type": "woql:Variable",
            "woql:variable_name": {"@value": "New", "@type": "xsd:string"},
        },
        "woql:query": {
            "@type": "woql:Triple",
            "woql:subject": {
                "@type": "woql:Variable",
                "woql:variable_name": {"@value": "A", "@type": "xsd:string"},
            },
            "woql:predicate": {
                "@type": "woql:Variable",
                "woql:variable_name": {"@value": "B", "@type": "xsd:string"},
            },
            "woql:object": {
                "@type": "woql:Variable",
                "woql:variable_name": {"@value": "C", "@type": "xsd:string"},
            },
        },
    },
    "addClassJson": {
        "@type": "woql:AddQuad",
        "woql:subject": {"@type": "woql:Node", "woql:node": "scm:id"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
        "woql:object": {"@type": "woql:Node", "woql:node": "owl:Class"},
        "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
    },
    "subsumptionJson": {
        "@type": "woql:Subsumption",
        "woql:parent": {"@type": "woql:Node", "woql:node": "scm:ClassA"},
        "woql:child": {"@type": "woql:Node", "woql:node": "scm:ClassB"},
    },
    "orderbyJson": {
        "@type": "woql:OrderBy",
        "woql:query": WOQL_STAR,
        "woql:variable_ordering": [
            {
                "@type": "woql:VariableOrdering",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:variable": {
                    "@type": "woql:Variable",
                    "woql:variable_name": {"@value": "A", "@type": "xsd:string"},
                },
            },
            {
                "@type": "woql:VariableOrdering",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:ascending": {"@type": "xsd:boolean", "@value": True},
                "woql:variable": {
                    "@type": "woql:Variable",
                    "woql:variable_name": {"@value": "B", "@type": "xsd:string"},
                },
            },
            {
                "@type": "woql:VariableOrdering",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 2},
                "woql:ascending": {"@type": "xsd:boolean", "@value": True},
                "woql:variable": {
                    "@type": "woql:Variable",
                    "woql:variable_name": {"@value": "C", "@type": "xsd:string"},
                },
            },
        ],
    },
    "isAJson": {
        "@type": "woql:IsA",
        "woql:element": {"@type": "woql:Node", "woql:node": "doc:instance"},
        "woql:of_type": {"@type": "woql:Node", "woql:node": "owl:Class"},
    },
    "deleteTripleJson": {
        "@type": "woql:DeleteTriple",
        "woql:subject": {"@type": "woql:Node", "woql:node": "doc:a"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:b"},
        "woql:object": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:string", "@value": "c"},
        },
    },
    "deleteQuadJson": {
        "@type": "woql:DeleteQuad",
        "woql:subject": {"@type": "woql:Node", "woql:node": "doc:a"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:b"},
        "woql:object": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:string", "@value": "c"},
        },
        "woql:graph": {"@type": "xsd:string", "@value": "d"},
    },
    "addTripleJson": {
        "@type": "woql:AddTriple",
        "woql:subject": {"@type": "woql:Node", "woql:node": "doc:a"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:b"},
        "woql:object": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:string", "@value": "c"},
        },
    },
    "addQuadJson": {
        "@type": "woql:AddQuad",
        "woql:subject": {"@type": "woql:Node", "woql:node": "doc:a"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:b"},
        "woql:object": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:string", "@value": "c"},
        },
        "woql:graph": {"@type": "xsd:string", "@value": "d"},
    },
    "addPropertyJson": {
        "@type": "woql:And",
        "woql:query_list": [
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {
                        "@type": "woql:Node",
                        "woql:node": "scm:some_property",
                    },
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "owl:DatatypeProperty",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {
                        "@type": "woql:Node",
                        "woql:node": "scm:some_property",
                    },
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:range"},
                    "woql:object": {"@type": "woql:Node", "woql:node": "xsd:string"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
        ],
    },
    "deletePropertyJson": {
        "@type": "woql:And",
        "woql:query_list": [
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:query": {
                    "@type": "woql:DeleteQuad",
                    "woql:subject": {
                        "@type": "woql:Node",
                        "woql:node": "scm:some_property",
                    },
                    "woql:predicate": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {"@value": "All", "@type": "xsd:string"},
                    },
                    "woql:object": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {"@value": "Al2", "@type": "xsd:string"},
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "string"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:query": {
                    "@type": "woql:DeleteQuad",
                    "woql:subject": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {"@value": "Al3", "@type": "xsd:string"},
                    },
                    "woql:predicate": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {"@value": "Al4", "@type": "xsd:string"},
                    },
                    "woql:object": {
                        "@type": "woql:Node",
                        "woql:node": "scm:some_property",
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "string"},
                },
            },
        ],
    },
    "graphMethodJson": {
        "@type": "woql:AddQuad",
        "woql:subject": {"@type": "woql:Node", "woql:node": "doc:x"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:label"},
        "woql:object": {
            "@type": "woql:Datatype",
            "woql:datatype": {
                "@value": "my label",
                "@type": "xsd:string",
                "@language": "en",
            },
        },
        "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
    },
    "labelMethodJson": {
        "@type": "woql:AddQuad",
        "woql:subject": {"@type": "woql:Node", "woql:node": "doc:x"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:label"},
        "woql:object": {
            "@type": "woql:Datatype",
            "woql:datatype": {
                "@value": "my label",
                "@type": "xsd:string",
                "@language": "en",
            },
        },
        "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
    },
    "labelMethodJson2": {
        "@type": "woql:AddQuad",
        "woql:subject": {"@type": "woql:Node", "woql:node": "doc:x"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:label"},
        "woql:object": {
            "@type": "woql:Variable",
            "woql:variable_name": {"@value": "label", "@type": "xsd:string"},
        },
        "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
    },
    "addClassDescJson": {
        "@type": "woql:And",
        "woql:query_list": [
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:NewClass"},
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {"@type": "woql:Node", "woql:node": "owl:Class"},
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:query": {
                    "@type": "woql:AddQuad",
                    "woql:subject": {"@type": "woql:Node", "woql:node": "scm:NewClass"},
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:comment",
                    },
                    "woql:object": {
                        "@type": "woql:Datatype",
                        "woql:datatype": {
                            "@value": "A new class object.",
                            "@type": "xsd:string",
                            "@language": "en",
                        },
                    },
                    "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
                },
            },
        ],
    },
    "addNodePropJson": {
        "@type": "woql:AddTriple",
        "woql:subject": {"@type": "woql:Node", "woql:node": "doc:x"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:myprop"},
        "woql:object": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:string", "@value": "value"},
        },
    },
    "nodeParentJson": {
        "@type": "woql:AddQuad",
        "woql:subject": {"@type": "woql:Node", "woql:node": "doc:x"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:subClassOf"},
        "woql:object": {"@type": "woql:Node", "woql:node": "scm:classParentName"},
        "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
    },
    "nodeAbstractJson": {
        "@type": "woql:AddQuad",
        "woql:subject": {"@type": "woql:Node", "woql:node": "doc:x"},
        "woql:predicate": {"@type": "woql:Node", "woql:node": "terminus:tag"},
        "woql:object": {"@type": "woql:Node", "woql:node": "terminus:abstract"},
        "woql:graph": {"@type": "xsd:string", "@value": "schema/main"},
    },
}



# read the contents of your README file
from os import path

import setuptools

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

requires = ["requests"]

test_requirements = [
    "pytest>=3",
    "pytest-cov",
    "pytest-mock",
    "pytest-xdist",
    "pytest-runner",
]

extras_require = {"dataframe": ["numpy >= 1.13.0", "pandas >= 0.23.0"]}

setuptools.setup(
    name="terminusdb-client",
    version="0.1.5",
    author="TerminusDB group",
    author_email="terminusdatabase@gmail.com",
    description="Python client for Terminus DB",
    long_description=long_description,
    long_description_content_type="text/markdown",
    package_data={"": ["LICENSE"]},
    url="https://github.com/terminusdb/terminusdb-client-python",
    packages=setuptools.find_packages(exclude=["tests"]),
    include_package_data=True,
    install_requires=requires,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    setup_requires=["pytest-runner"],
    tests_require=test_requirements,
    extras_require=extras_require,
    python_requires=">=3.6",
)



# idParser.py


class IDParser:
    def __init__(self):
        pass

    def _valid_url(self, input_str):
        if input_str is None:
            return False
        if input_str[:7] == "http://" or input_str[:8] == "https://":
            return True
        return False

    def _valid_id_string(self, input_str, _=None):
        if type(input_str) != str:
            return False
        if (":" in input_str) or (" " in input_str) or ("/" in input_str):
            return False
        return True

    def parse_server_url(self, url):
        if not self._valid_url(url):
            return False
        if url[-1] != "/":
            url += "/"
        return url

    def parse_dbid(self, dbid):
        if self._valid_id_string(dbid, "db"):
            return dbid
        return False

    def parse_account(self, account):
        if self._valid_id_string(account, "account"):
            return account
        return False

    def parse_branch(self, bid):
        if self._valid_id_string(bid, "branch"):
            return bid
        return False

    def parse_jwt(self, jwt):
        return jwt

    def parse_key(self, key):
        return key



class DocumentTemplate:
    def __init__(self):
        pass

    @staticmethod
    def create_db_template(server, db_id, label, **kwargs):
        db_url = "{}{}".format(server, db_id)
        comment = kwargs.get("comment")
        language = kwargs.get("language", "en")
        allow_origin = kwargs.get("allow_origin", "*")

        temp = {
            "@context": {
                "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
                "terminus": "http://terminusdb.com/schema/terminus#",
                "_": db_url,
            },
            "terminus:document": {
                "@type": "terminus:Database",
                "rdfs:label": {"@language": language, "@value": label},
                "terminus:allow_origin": {
                    "@type": "xsd:string",
                    "@value": allow_origin,
                },
                "@id": db_url,
            },
            "@type": "terminus:APIUpdate",
        }
        if comment:
            temp["rdfs:comment"] = {"@language": language, "@value": comment}
        return temp

        @staticmethod
        def format_document(doc, schema_url, options=None, url_id=None):
            document = {}
            if isinstance(doc, dict):
                document["@context"] = doc["@context"]
                # add blank node prefix as document base url
                if ("@context" in doc) and ("@id" in doc):
                    document["@context"]["_"] = doc["@id"]

                if (
                    options
                    and options.get("terminus:encoding")
                    and options["terminus:encoding"] == "terminus:turtle"
                ):
                    document["terminus:turtle"] = doc
                    # document['terminus:schema'] = schema_url
                    del document["terminus:turtle"]["@context"]
                else:
                    document["terminus:document"] = doc
                    del document["terminus:document"]["@context"]

                document["@type"] = "terminus:APIUpdate"

                if url_id:
                    document["@id"] = url_id

            return document



WOQL_JOIN_SPLIT_JSON = {
    "joinJson": {
        "@type": "woql:Join",
        "woql:join_list": {
            "@type": "woql:Array",
            "woql:array_element": [
                {
                    "@type": "woql:ArrayElement",
                    "woql:variable_name": {"@value": "A_obj", "@type": "xsd:string"},
                    "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                },
                {
                    "@type": "woql:ArrayElement",
                    "woql:variable_name": {"@value": "B_obj", "@type": "xsd:string"},
                    "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                },
            ],
        },
        "woql:join_separator": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:string", "@value": ", "},
        },
        "woql:join": {
            "@type": "woql:Variable",
            "woql:variable_name": {"@value": "output", "@type": "xsd:string"},
        },
    },
    "splitJson": {
        "@type": "woql:Split",
        "woql:split_string": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:string", "@value": "A, B, C"},
        },
        "woql:split_pattern": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:string", "@value": ", "},
        },
        "woql:split_list": {
            "@type": "woql:Variable",
            "woql:variable_name": {"@value": "list_obj", "@type": "xsd:string"},
        },
    },
}



# woqlDataframe.py

import warnings

try:
    import numpy as np
    import pandas as pd
except ImportError:
    msg = (
        "woqlDataframe requirements are not installed.\n\n"
        "If you want to use woqlDataframe, please pip install as follows:\n\n"
        "  python -m pip install -U terminus-client-python[dataframe]"
    )
    warnings.warn(msg)


class EmptyException(Exception):
    pass


def _is_empty(query):
    return len(query["bindings"]) == 0


def extract_header(query):
    """Extracts the header of the returned result table

    Extrace the result of the query binding, including the header name and their WOQL rdf data type.

    Parameters
    ----------
    query : dict
            JSON of the result of the query

    Returns
    -------
    list
        List of tuples, each tuple will have the name of the column as the 1st element an the WOQL rdf data type of that column as the 2nd element.

    Examples
    --------
    >>> result = {'bindings': [{
    ...           'Product': {'@type': 'http://www.w3.org/2001/XMLSchema#string',
    ...           '@value': 'STRAWBERRY CANDY'}
    ...           }], "graphs": []}
    >>> woql.extract_header(result)
    [('Product', 'http://www.w3.org/2001/XMLSchema#string')]

    See Also
    --------
    query_to_df : put the result of the query into a pandas DataFrame
    type_map : convert WQOL rdf data types into numpy data types
    extract_column : extract the column of the returned result table
    """
    header = []
    if _is_empty(query):
        raise EmptyException("Query is empty")

    bindings = query["bindings"][0]
    for name, v in bindings.items():
        if type(v) == dict and ("@type" in v):
            ty = v["@type"]
        else:
            ty = "http://www.w3.org/2001/XMLSchema#string"
        header.append((name, ty))
    return header


def extract_column(query, name, ty):
    """Extracts the column of the returned result table

    Extrace one specific column from the result of the query binding.

    Parameters
    ----------
    query : dict
            JSON of the result of the query
    name : str
            Name of the column, can be extract from `extract_header()`
    ty : str
            WOQL rdf type of the column

    Returns
    -------
    list
        List consist of elements in that column

    Examples
    --------
    >>> result = {'bindings': [{
    ...           Product': {'@type': 'http://www.w3.org/2001/XMLSchema#string',
    ...           '@value': 'STRAWBERRY CANDY'}
    ...           }], "graphs": []}
    >>> woql.extract_column(result, 'Product', 'http://www.w3.org/2001/XMLSchema#string')
    ['STRAWBERRY CANDY']

    See Also
    --------
    query_to_df : put the result of the query into a pandas DataFrame
    type_map : convert WQOL rdf data types into numpy data types
    extract_header : extract header of the returned result table
    """
    bindings = query["bindings"]
    column = []
    for binding in bindings:
        for k, v in binding.items():
            if k == (name):
                if isinstance(v, dict):
                    value = type_value_map(ty, v["@value"])
                else:
                    value = type_value_map(ty, v)
                column.append(value)
    return column


def type_map(ty_rdf):
    """Mapping types from WOQL rdf to numpy data types

    Maps the WOQL rdf type in the result query binding to numpy data types so it can be used in constructed pandas DataFrame.

    Parameters
    ----------
    ty_rdf : str
            WOQL rdf type, an exception will be raise if it's not a valid type

    Returns
    -------
    one of the numpy built-in scalar types objects
        The numpy data type object that matches with the WOQL rdf data type

    Examples
    --------
    >>> woql.type_map('http://www.w3.org/2001/XMLSchema#string')
    <class 'numpy.str_'>

    See Also
    --------
    query_to_df : put the result of the query into a pandas DataFrame
    type_value_map : converts values of different WOQL rdf types to numpy data types values
    """
    convert_mapping = {
        "http://www.w3.org/2001/XMLSchema#string": np.unicode_,
        "http://www.w3.org/2001/XMLSchema#integer": np.int,
        "http://www.w3.org/2001/XMLSchema#dateTime": np.datetime64,
        "http://www.w3.org/2001/XMLSchema#decimal": np.double,
    }
    if ty_rdf in convert_mapping:
        return convert_mapping[ty_rdf]
    else:
        raise Exception("Unknown rdf type! " + ty_rdf)


def type_value_map(ty_rdf, value):
    """Converts values of different WOQL rdf types to numpy data types values

    Converts the values in the result query binding to numpy data types according to their WOQL rdf type, so it can be used in constructed pandas DataFrame.

    Parameters
    ----------
    ty_rdf : str
            WOQL rdf type, an exception will be raise if it's not a valid type
    value : str
            value to be converted

    Returns
    -------
    one of the numpy built-in scalar types
        The converted value in numpy data type

    Examples
    --------
    >>> woql.type_value_map('http://www.w3.org/2001/XMLSchema#decimal', '10.80')
    10.8

    See Also
    --------
    query_to_df : put the result of the query into a pandas DataFrame
    type_map : mapping types from WOQL rdf to numpy data types
    """
    if ty_rdf == "http://www.w3.org/2001/XMLSchema#string":
        return value
    elif ty_rdf == "http://www.w3.org/2001/XMLSchema#integer":
        return int(value)
    elif ty_rdf == "http://www.w3.org/2001/XMLSchema#dateTime":
        return np.datetime64(value)
    elif ty_rdf == "http://www.w3.org/2001/XMLSchema#decimal":
        return float(value)
    else:
        raise Exception("Unknown rdf type! " + ty_rdf)


def query_to_df(query):
    """Convert a query to a data frame.

    Converts result query binding to a pandas DataFrame. This only works for homogeneous query results

    Parameters
    ----------
    query : dict
            JSON of the result of the query

    Returns
    -------
    pandas.DataFrame
        The pandas DataFrame that is converted from the result.

    Examples
    --------
    >>> result = {'bindings': [{
    ...           'http://terminusdb.com/woql/variable/Product': {'@type': 'http://www.w3.org/2001/XMLSchema#string',
    ...           '@value': 'STRAWBERRY CANDY'}
    ...           }], "graphs": []}
    >>> woql.query_to_df(result)
                Product
    0  STRAWBERRY CANDY

    See Also
    --------
    WOQLQuery : create a WOQLQuery
    WOQLClient : create a WOQLClient
    """
    header = extract_header(query)
    dtypes = {}
    column_names = []
    for name, rdftype in header:
        column_names.append(name)
        dtype = type_map(rdftype)
        dtypes[name] = dtype

    dataframe = pd.DataFrame(columns=column_names)
    dataframe.astype(dtypes)

    for name, rdftype in header:
        column = extract_column(query, name, rdftype)
        dataframe[name] = column

    return dataframe



__title__ = "woqlClient"
__description__ = "woqlClient library for accessing the Terminus DB API"
__url__ = ""
__version__ = "0.1.5"
__build__ = 00
__author__ = "TerminusDB group"
__author_email__ = "terminusdatabase@gmail.com"
__license__ = "GNU General Public License v3 (GPLv3)"
__copyright__ = "Copyright 2019 datachemist"



import copy
import pprint

# import pprint
import unittest.mock as mock

import requests
from terminusdb_client.woqlclient.woqlClient import WOQLClient
from terminusdb_client.woqlquery.woql_query import WOQLQuery

from .ans_cardinality import *  # noqa
from .ans_doctype import *  # noqa
from .ans_insert import *  # noqa
from .ans_property import *  # noqa
from .ans_triple_quad import *  # noqa
from .mockResponse import mocked_requests
from .test_connectionCapabilities import json_context

# expected results
from .woqljson.woqlAndJson import WOQL_AND_JSON
from .woqljson.woqlCastJson import WOQL_CAST_JSON
from .woqljson.woqlConcatJson import WOQL_CONCAT_JSON
from .woqljson.woqlDeleteJson import WOQL_DELETE_JSON
from .woqljson.woqlIdgenJson import WOQL_IDGEN_JSON
from .woqljson.woqlJoinSplitJson import WOQL_JOIN_SPLIT_JSON
from .woqljson.woqlJson import WOQL_JSON, WOQL_STAR
from .woqljson.woqlMathJson import WOQL_MATH_JSON
from .woqljson.woqlOrJson import WOQL_OR_JSON
from .woqljson.woqlTrimJson import WOQL_TRIM_JSON
from .woqljson.woqlWhenJson import WOQL_WHEN_JSON

pp = pprint.PrettyPrinter(indent=4)


class TestWoqlQueries:
    @mock.patch("requests.post", side_effect=mocked_requests)
    @mock.patch("requests.get", side_effect=mocked_requests)
    def test_execute_method(self, mocked_requests, mocked_requests2):
        woql_client = WOQLClient("http://localhost:6363")
        woql_client.connect(user="admin", account="admin", key="root", db="myDBName")
        woql_object = WOQLQuery().star()
        woql_object.execute(woql_client)

        mycon = copy.deepcopy(WOQL_STAR)  # TODO: make WOQL_STAR immutable
        mycon["@context"] = json_context

        requests.post.assert_called_once_with(
            "http://localhost:6363/woql/admin/myDBName/local/branch/master",
            headers={
                "Authorization": "Basic YWRtaW46cm9vdA==",
                "content-type": "application/json",
            },
            json={"query": mycon},
        )

    def test_start_properties_values(self):
        woql_object = WOQLQuery()
        assert not woql_object._chain_ended
        assert not woql_object._contains_update
        assert woql_object._vocab["type"] == "rdf:type"

    def test_limit_method(self):
        woql_object = WOQLQuery().limit(10)
        limit_json = {}
        assert woql_object.to_dict() == limit_json

    def test_start_method(self):
        woql_object = WOQLQuery().limit(10).start(0).star()
        json_obj = {
            "@type": "woql:Limit",
            "woql:limit": {
                "@type": "woql:Datatype",
                "woql:datatype": {"@type": "xsd:nonNegativeInteger", "@value": 10},
            },
            "woql:query": {
                "@type": "woql:Start",
                "woql:query": WOQL_STAR,
                "woql:start": {
                    "@type": "woql:Datatype",
                    "woql:datatype": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                },
            },
        }
        assert woql_object.to_dict() == json_obj

    def test_insert_method(self, insert_without, insert_with_des):
        woql_object = WOQLQuery().insert("v:Bike_URL", "Bicycle")
        woql_object_des = WOQLQuery().insert(
            "v:Bike_URL", "Bicycle", label="v:Bike_Label", description="v:Bike_Des"
        )
        assert woql_object.to_dict() == insert_without
        assert woql_object_des.to_dict() == insert_with_des

    def test_doctype_method(
        self, doctype_without, doctype_with_label, doctype_with_des
    ):
        woql_object = WOQLQuery().doctype("Station")
        woql_object_label = WOQLQuery().doctype("Station", label="Station Object")
        woql_object_des = WOQLQuery().doctype(
            "Station", label="Station Object", description="A bike station object."
        )
        assert woql_object.to_dict() == doctype_without
        assert woql_object_label.to_dict() == doctype_with_label
        assert woql_object_des.to_dict() == doctype_with_des

    def test_doctype_property_method(self, property_without, property_with_des):
        woql_object = WOQLQuery().doctype("Journey").property("Duration", "dateTime")
        woql_object_des = (
            WOQLQuery()
            .doctype("Journey")
            .property(
                "Duration",
                "dateTime",
                label="Journey Duration",
                description="Journey duration in minutes.",
            )
        )

        pp.pprint(woql_object_des.to_dict())
        assert woql_object.to_dict() == property_without
        assert woql_object_des.to_dict() == property_with_des

    def test_woql_not_method(self):
        woql_object = WOQLQuery().woql_not(WOQLQuery().triple("a", "b", "c"))
        woql_object_chain = WOQLQuery().woql_not()
        woql_object_chain.triple("a", "b", "c")
        json_obj = {
            "@type": "woql:Not",
            "woql:query": {
                "@type": "woql:Triple",
                "woql:subject": {"@type": "woql:Node", "woql:node": "doc:a"},
                "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:b"},
                "woql:object": {
                    "@type": "woql:Datatype",
                    "woql:datatype": {"@type": "xsd:string", "@value": "c"},
                },
            },
        }
        assert woql_object.to_dict() == json_obj
        assert woql_object_chain.to_dict() == json_obj

    def test_woql_and_method(self):
        woql_object = WOQLQuery().woql_and(
            WOQLQuery().triple("a", "b", "c"), WOQLQuery().triple("1", "2", "3")
        )

        assert woql_object.to_dict() == WOQL_AND_JSON

    def test_woql_or_method(self):
        woql_object = WOQLQuery().woql_or(
            WOQLQuery().triple("a", "b", "c"), WOQLQuery().triple("1", "2", "3")
        )
        assert woql_object.to_dict() == WOQL_OR_JSON

    def test_when_method(self):

        woql_object = WOQLQuery().when(True, WOQLQuery().add_class("id"))
        woql_object_chain = WOQLQuery().when(True).add_class("id")
        assert woql_object.to_dict() == WOQL_WHEN_JSON
        assert woql_object_chain.to_dict() == WOQL_WHEN_JSON

    def test_opt_method(self):
        woql_object = WOQLQuery().opt(WOQLQuery().star())
        woql_object_chain = WOQLQuery().opt().star()
        json_obj = {"@type": "woql:Optional", "woql:query": WOQL_STAR}
        assert woql_object.to_dict() == json_obj
        assert woql_object_chain.to_dict() == json_obj

    def test_woql_from_method(self):
        woql_object = WOQLQuery().woql_from("schema/main", WOQLQuery().star())
        woql_object_chain = WOQLQuery().woql_from("schema/main").star()
        json_obj = {
            "@type": "woql:From",
            "woql:graph_filter": {"@type": "xsd:string", "@value": "schema/main"},
            "woql:query": WOQL_STAR,
        }
        assert woql_object.to_dict() == woql_object_chain.to_dict()
        assert woql_object_chain.to_dict() == json_obj

    def test_star_method(self):
        woql_object = WOQLQuery().star()
        assert woql_object.to_dict() == WOQL_STAR

    def test_select_method(self):
        # TODO: chain won't work
        woql_object = WOQLQuery().select("V1", WOQLQuery().star())
        woql_object_multiple = WOQLQuery().select("V1", "V2", WOQLQuery().star())
        woql_object_chain = WOQLQuery().select("V1").star()
        woql_object_chain_multiple = WOQLQuery().select("V1", "V2").star()

        assert woql_object_chain_multiple.to_dict() == woql_object_multiple.to_dict()
        assert woql_object.to_dict() == woql_object_chain.to_dict()

    def test_eq_method(self):
        woql_object = WOQLQuery().eq("a", "b")
        json_obj = {
            "@type": "woql:Equals",
            "woql:left": {
                "@type": "woql:Datatype",
                "woql:datatype": {"@type": "xsd:string", "@value": "a"},
            },
            "woql:right": {
                "@type": "woql:Datatype",
                "woql:datatype": {"@type": "xsd:string", "@value": "b"},
            },
        }
        assert woql_object.to_dict() == json_obj

    def test_trim_method(self):
        woql_object = WOQLQuery().trim("a", "b")
        assert woql_object.to_dict() == WOQL_TRIM_JSON

    def test_eval_method(self):
        woql_object = WOQLQuery().eval("1+2", "b")
        assert woql_object.to_dict() == WOQL_MATH_JSON["evalJson"]

    def test_minus_method(self):

        woql_object = WOQLQuery().minus("2", "1")
        assert woql_object.to_dict() == WOQL_MATH_JSON["minusJson"]

    def test_plus_method(self):
        woql_object = WOQLQuery().plus("2", "1")
        assert woql_object.to_dict() == WOQL_MATH_JSON["plusJson"]

    def test_times_method(self):
        woql_object = WOQLQuery().times("2", "1")
        assert woql_object.to_dict() == WOQL_MATH_JSON["timesJson"]

    def test_divide_method(self):
        woql_object = WOQLQuery().divide("2", "1")
        assert woql_object.to_dict() == WOQL_MATH_JSON["divideJson"]

    def test_exp_method(self):
        woql_object = WOQLQuery().exp("2", "1")
        assert woql_object.to_dict() == WOQL_MATH_JSON["expJson"]

    def test_div_method(self):
        woql_object = WOQLQuery().div("2", "1")
        assert woql_object.to_dict() == WOQL_MATH_JSON["divJson"]

    def test_get_method(self):
        woql_object = WOQLQuery().get(WOQLQuery().woql_as("a", "b"), "Target")
        json_obj = {
            "@type": "woql:Get",
            "woql:as_vars": [
                {
                    "@type": "woql:NamedAsVar",
                    "woql:identifier": {"@type": "xsd:string", "@value": "a"},
                    "woql:variable_name": {"@type": "xsd:string", "@value": "b"},
                }
            ],
            "woql:query_resource": "Target",
        }
        assert woql_object.to_dict() == json_obj

    def test_woql_as_method(self):
        woql_object = WOQLQuery().woql_as("Source", "Target")
        woql_object2 = (
            WOQLQuery().woql_as("Source", "Target").woql_as("Source2", "Target2")
        )
        json_obj2 = [
            {
                "@type": "woql:NamedAsVar",
                "woql:identifier": {"@type": "xsd:string", "@value": "Source"},
                "woql:variable_name": {"@type": "xsd:string", "@value": "Target"},
            },
            {
                "@type": "woql:NamedAsVar",
                "woql:identifier": {"@type": "xsd:string", "@value": "Source2"},
                "woql:variable_name": {"@type": "xsd:string", "@value": "Target2"},
            },
        ]

        assert woql_object.to_dict() == [json_obj2[0]]
        assert woql_object2.to_dict() == json_obj2

    def test_remote_method(self):
        woql_object = WOQLQuery().remote({"url": "http://url"})
        json_obj = {
            "@type": "woql:RemoteResource",
            "woql:remote_uri": {"@type": "xsd:anyURI", "@value": {"url": "http://url"}},
        }
        assert woql_object.to_dict() == json_obj

    def test_post_method(self):
        woql_object = WOQLQuery().post("my_json_file", {"format": "panda_json"})
        json_obj = {
            "@type": "woql:PostResource",
            "woql:file": {"@type": "xsd:string", "@value": "my_json_file"},
            "woql:format": {
                "@type": "woql:Format",
                "woql:format_type": {"@value": "panda_json", "@type": "xsd:string"},
            },
        }
        assert woql_object.to_dict() == json_obj

    def test_idgen_method(self):
        woql_object = WOQLQuery().idgen(
            "doc:Station", "v:Start_ID", "v:Start_Station_URL"
        )
        assert woql_object.to_dict() == WOQL_IDGEN_JSON

    def test_typecast_method(self):
        woql_object = WOQLQuery().typecast(
            "v:Duration", "xsd:integer", "v:Duration_Cast"
        )
        json_obj = {
            "@type": "woql:Typecast",
            "woql:typecast_value": {
                "@type": "woql:Variable",
                "woql:variable_name": {"@value": "Duration", "@type": "xsd:string"},
            },
            "woql:typecast_type": {"@type": "woql:Node", "woql:node": "xsd:integer"},
            "woql:typecast_result": {
                "@type": "woql:Variable",
                "woql:variable_name": {
                    "@value": "Duration_Cast",
                    "@type": "xsd:string",
                },
            },
        }
        assert woql_object.to_dict() == json_obj

    def test_cast_method(self):
        woql_object = WOQLQuery().cast("v:Duration", "xsd:integer", "v:Duration_Cast")
        assert woql_object.to_dict() == WOQL_CAST_JSON

    def test_re_method(self):
        woql_object = WOQLQuery().re("!\\w+(.*)test$", "v:string", "v:formated")
        json_obj = {
            "@type": "woql:Regexp",
            "woql:pattern": {
                "@type": "woql:Datatype",
                "woql:datatype": {"@type": "xsd:string", "@value": "!\\w+(.*)test$"},
            },
            "woql:regexp_list": {
                "@type": "woql:Variable",
                "woql:variable_name": {"@type": "xsd:string", "@value": "formated"},
            },
            "woql:regexp_string": {
                "@type": "woql:Variable",
                "woql:variable_name": {"@type": "xsd:string", "@value": "string"},
            },
        }
        assert woql_object.to_dict() == json_obj

    def test_join_method(self):
        woql_object = WOQLQuery().join(["v:A_obj", "v:B_obj"], ", ", "v:output")
        assert woql_object.to_dict() == WOQL_JOIN_SPLIT_JSON["joinJson"]

    def test_split_method(self):
        woql_object = WOQLQuery().split("A, B, C", ", ", "v:list_obj")
        assert woql_object.to_dict() == WOQL_JOIN_SPLIT_JSON["splitJson"]

    def test_member_method(self):
        woql_object = WOQLQuery().member("v:member", "v:list_obj")
        json_obj = {
            "@type": "woql:Member",
            "woql:member": {
                "@type": "woql:Variable",
                "woql:variable_name": {"@value": "member", "@type": "xsd:string"},
            },
            "woql:member_list": {
                "@type": "woql:Variable",
                "woql:variable_name": {"@value": "list_obj", "@type": "xsd:string"},
            },
        }
        assert woql_object.to_dict() == json_obj

    def test_concat_method(self):
        woql_object = WOQLQuery().concat("v:Duration yo v:Duration_Cast", "v:x")
        assert woql_object.to_dict() == WOQL_CONCAT_JSON

    def test_group_by_method(self):
        woql_object = (
            WOQLQuery()
            .group_by(["v:A", "v:B"], ["v:C"], "v:New")
            .triple("v:A", "v:B", "v:C")
        )
        assert woql_object.to_dict() == WOQL_JSON["groupbyJson"]

    def test_order_by_method_asc(self):
        woql_object = WOQLQuery().order_by("v:A", "v:B asc", "v:C asc").star()
        assert woql_object.to_dict() == WOQL_JSON["orderbyJson"]


class TestTripleBuilder:
    def test_triple_method(self, triple_opt):
        woql_object = WOQLQuery().triple("a", "b", "c")
        woql_object_opt = WOQLQuery().triple("a", "b", "c", opt=True)
        assert woql_object.to_dict() == WOQL_JSON["tripleJson"]
        assert woql_object_opt.to_dict() == triple_opt

    def test_quad_method(self, quad_opt):
        woql_object = WOQLQuery().quad("a", "b", "c", "d")
        woql_object_opt = WOQLQuery().quad("a", "b", "c", "d", opt=True)
        assert woql_object.to_dict() == WOQL_JSON["quadJson"]
        assert woql_object_opt.to_dict() == quad_opt

    def test_add_class_method(self):
        woql_object = WOQLQuery().add_class("id")
        assert woql_object.to_dict() == WOQL_JSON["addClassJson"]

    def test_delete_class_method(self):
        woql_object = WOQLQuery().delete_class("id")
        assert woql_object.to_dict() == WOQL_DELETE_JSON

    def test_sub_method(self):
        woql_object = WOQLQuery().sub("ClassA", "ClassB")
        assert woql_object.to_dict() == WOQL_JSON["subsumptionJson"]

    def test_isa_method(self):
        woql_object = WOQLQuery().isa("instance", "Class")
        assert woql_object.to_dict() == WOQL_JSON["isAJson"]

    def test_delete_method(self):
        woql_object = WOQLQuery().delete_object("doc:x")
        json_obj = {"@type": "woql:DeleteObject", "woql:document_uri": "doc:x"}
        assert woql_object.to_dict() == json_obj

    def test_delete_triple_method(self):
        woql_object = WOQLQuery().delete_triple("a", "b", "c")
        assert woql_object.to_dict() == WOQL_JSON["deleteTripleJson"]

    def test_delete_quad_method(self):
        woql_object = WOQLQuery().delete_quad("a", "b", "c", "d")
        assert woql_object.to_dict() == WOQL_JSON["deleteQuadJson"]

    def test_add_triple_method(self):
        woql_object = WOQLQuery().add_triple("a", "b", "c")
        assert woql_object.to_dict() == WOQL_JSON["addTripleJson"]

    def test_add_quad_method(self):
        woql_object = WOQLQuery().add_quad("a", "b", "c", "d")
        assert woql_object.to_dict() == WOQL_JSON["addQuadJson"]

    def test_add_property_method(self):
        woql_object = WOQLQuery().add_property("some_property", "string")
        assert woql_object.to_dict() == WOQL_JSON["addPropertyJson"]

    def test_delete_property_method(self):
        woql_object = WOQLQuery().delete_property("some_property", "string")
        assert woql_object.to_dict() == WOQL_JSON["deletePropertyJson"]


class TestTripleBuilderChainer:
    def test_node_method(self):
        woql_object = WOQLQuery().node("some_node", "add_quad")
        assert woql_object.to_dict() == {}

    def test_graph_method(self):
        woql_object = WOQLQuery().node("a", "AddQuad").graph("schema/main")
        woql_object2 = (
            WOQLQuery()
            .node("doc:x", "add_quad")
            .graph("schema/main")
            .label("my label", "en")
        )

        assert woql_object.to_dict() == {}
        assert woql_object2.to_dict() == WOQL_JSON["graphMethodJson"]

    def test_node_and_label_method(self):
        woql_object = WOQLQuery().node("doc:x", "add_quad").label("my label", "en")
        woql_object2 = WOQLQuery().node("doc:x", "add_quad").label("v:label")
        pp.pprint(woql_object.to_dict())
        assert woql_object.to_dict() == WOQL_JSON["labelMethodJson"]
        assert woql_object2.to_dict() == WOQL_JSON["labelMethodJson2"]

    def test_add_class_and_description_method(self):
        woql_object = (
            WOQLQuery().add_class("NewClass").description("A new class object.")
        )
        assert woql_object.to_dict() == WOQL_JSON["addClassDescJson"]

    def test_comment_method(self):
        woql_object = WOQLQuery().comment("my comment")
        json_obj = {
            "@type": "woql:Comment",
            "woql:comment": {"@type": "xsd:string", "@value": "my comment"},
        }
        assert woql_object.to_dict() == json_obj

    def test_abstract_method(self):
        woql_object = WOQLQuery().node("doc:x", "add_quad").abstract()
        assert woql_object.to_dict() == WOQL_JSON["nodeAbstractJson"]

    def test_node_property_method(self):
        woql_object = (
            WOQLQuery().node("doc:x", "add_triple").property("myprop", "value")
        )
        assert woql_object.to_dict() == WOQL_JSON["addNodePropJson"]

    def test_node_parent_method(self):
        woql_object = WOQLQuery().node("doc:x", "add_quad").parent("classParentName")
        assert woql_object.to_dict() == WOQL_JSON["nodeParentJson"]

    def test_class_description_method(self):
        woql_object = (
            WOQLQuery().add_class("NewClass").description("A new class object.")
        )
        assert woql_object.to_dict() == WOQL_JSON["addClassDescJson"]

    def test_max(self, property_max):
        woql_object = WOQLQuery().add_property("P", "string").domain("A").max(4)
        assert woql_object.to_dict() == property_max

    def test_min(self, property_min):
        woql_object = WOQLQuery().add_property("P", "string").domain("A").min(2)
        assert woql_object.to_dict() == property_min

    def test_card(self, property_cardinalty):
        woql_object = WOQLQuery().add_property("P", "string").domain("A").cardinality(3)
        assert woql_object.to_dict() == property_cardinalty

    def test_chain1(self, chain_insert):
        woql_object = (
            WOQLQuery()
            .insert("v:Node_ID", "v:Type")
            .label("v:Label")
            .description("v:Description")
            .property("prop", "v:Prop")
            .property("prop", "v:Prop2")
            .parent("myParentClass")
        )
        assert woql_object.to_dict() == chain_insert

    def test_chain2(self, chain_doctype):
        woql_object = (
            WOQLQuery()
            .doctype("MyDoc")
            .label("abc")
            .description("abcd")
            .property("prop", "dateTime")
            .label("aaa")
            .property("prop2", "integer")
            .label("abe")
        )
        assert woql_object.to_dict() == chain_doctype

    def test_dot_chain(self):
        woql_object = WOQLQuery().triple("A", "B", "C").triple("D", "E", "F")
        v2 = WOQLQuery().woql_and(
            WOQLQuery().triple("A", "B", "C"), WOQLQuery().triple("D", "E", "F")
        )
        v3 = WOQLQuery().triple("A", "B", "C").woql_and().triple("D", "E", "F")
        assert woql_object.to_dict() == v2.to_dict()
        assert woql_object.to_dict() == v3.to_dict()



WOQL_MATH_JSON = {
    "divJson": {
        "@type": "woql:Div",
        "woql:first": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:decimal", "@value": "2"},
        },
        "woql:second": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:decimal", "@value": "1"},
        },
    },
    "minusJson": {
        "@type": "woql:Minus",
        "woql:first": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:decimal", "@value": "2"},
        },
        "woql:second": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:decimal", "@value": "1"},
        },
    },
    "evalJson": {
        "@type": "woql:Eval",
        "woql:expression": "1+2",
        "woql:result": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:string", "@value": "b"},
        },
    },
    "plusJson": {
        "@type": "woql:Plus",
        "woql:first": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:decimal", "@value": "2"},
        },
        "woql:second": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:decimal", "@value": "1"},
        },
    },
    "timesJson": {
        "@type": "woql:Times",
        "woql:first": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:decimal", "@value": "2"},
        },
        "woql:second": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:decimal", "@value": "1"},
        },
    },
    "divideJson": {
        "@type": "woql:Divide",
        "woql:first": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:decimal", "@value": "2"},
        },
        "woql:second": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:decimal", "@value": "1"},
        },
    },
    "expJson": {
        "@type": "woql:Exp",
        "woql:first": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:decimal", "@value": "2"},
        },
        "woql:second": {
            "@type": "woql:Datatype",
            "woql:datatype": {"@type": "xsd:decimal", "@value": "1"},
        },
    },
}



WOQL_AND_JSON = {
    "@type": "woql:And",
    "woql:query_list": [
        {
            "@type": "woql:QueryListElement",
            "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
            "woql:query": {
                "@type": "woql:Triple",
                "woql:subject": {"@type": "woql:Node", "woql:node": "doc:a"},
                "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:b"},
                "woql:object": {
                    "@type": "woql:Datatype",
                    "woql:datatype": {"@type": "xsd:string", "@value": "c"},
                },
            },
        },
        {
            "@type": "woql:QueryListElement",
            "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
            "woql:query": {
                "@type": "woql:Triple",
                "woql:subject": {"@type": "woql:Node", "woql:node": "doc:1"},
                "woql:predicate": {"@type": "woql:Node", "woql:node": "scm:2"},
                "woql:object": {
                    "@type": "woql:Datatype",
                    "woql:datatype": {"@type": "xsd:string", "@value": "3"},
                },
            },
        },
    ],
}



import pytest


@pytest.fixture(scope="module")
def insert_without():
    return {
        "@type": "woql:AddTriple",
        "woql:subject": {
            "@type": "woql:Variable",
            "woql:variable_name": {"@value": "Bike_URL", "@type": "xsd:string"},
        },
        "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
        "woql:object": {"@type": "woql:Node", "woql:node": "scm:Bicycle"},
    }


@pytest.fixture(scope="module")
def insert_with_des():
    return {
        "@type": "woql:And",
        "woql:query_list": [
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 0},
                "woql:query": {
                    "@type": "woql:AddTriple",
                    "woql:subject": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Bike_URL",
                            "@type": "xsd:string",
                        },
                    },
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdf:type"},
                    "woql:object": {"@type": "woql:Node", "woql:node": "scm:Bicycle"},
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1},
                "woql:query": {
                    "@type": "woql:AddTriple",
                    "woql:subject": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Bike_URL",
                            "@type": "xsd:string",
                        },
                    },
                    "woql:predicate": {"@type": "woql:Node", "woql:node": "rdfs:label"},
                    "woql:object": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Bike_Label",
                            "@type": "xsd:string",
                        },
                    },
                },
            },
            {
                "@type": "woql:QueryListElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 2},
                "woql:query": {
                    "@type": "woql:AddTriple",
                    "woql:subject": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Bike_URL",
                            "@type": "xsd:string",
                        },
                    },
                    "woql:predicate": {
                        "@type": "woql:Node",
                        "woql:node": "rdfs:comment",
                    },
                    "woql:object": {
                        "@type": "woql:Variable",
                        "woql:variable_name": {
                            "@value": "Bike_Des",
                            "@type": "xsd:string",
                        },
                    },
                },
            },
        ],
    }



WOQL_IDGEN_JSON = {
    "@type": "woql:IDGenerator",
    "woql:base": {
        "@type": "woql:Datatype",
        "woql:datatype": {"@type": "xsd:string", "@value": "doc:Station"},
    },
    "woql:key_list": {
        "@type": "woql:Array",
        "woql:array_element": [
            {
                "@type": "woql:ArrayElement",
                "woql:index": {"@type": "xsd:nonNegativeInteger", "@value": 1 - 1},
                "woql:variable_name": {"@type": "xsd:string", "@value": "Start_ID"},
            }
        ],
    },
    "woql:uri": {
        "@type": "woql:Variable",
        "woql:variable_name": {"@type": "xsd:string", "@value": "Start_Station_URL"},
    },
}



WoqlRe = {
    "@type": "woql:Regexp",
    "woql:pattern": {
        "@type": "woql:Datatype",
        "woql:datatype": {"@type": "xsd:string", "@value": ".*"},
    },
    "woql:regexp_string": {
        "@type": "woql:Variable",
        "woql:variable_name": {"@value": "string", "@type": "xsd:string"},
    },
    "woql:regexp_list": {
        "@type": "woql:Variable",
        "woql:variable_name": {"@value": "formated", "@type": "xsd:string"},
    },
}



# errorMessage.py


class ErrorMessage:
    def __init__(self):
        pass

    @staticmethod
    def get_error_as_message(url, api, err):
        message = "Code: " + str(err["status"])
        if "body" in err:
            message += ", Message: " + err["body"]
        if "action" in err:
            message += ", Action: " + err["action"]
        if "type" in err:
            message += ", Type: " + err["type"]
        if url:
            message += ", url: " + url
        if api and "method" in api:
            message += ", method: " + api["method"]
        return message

    @staticmethod
    def access_denied_err_obj(action, db, server):
        err = {}
        err["status"] = 403
        err["url"] = (server or "") + (db or "")
        err["type"] = "client"
        err["action"] = action
        err["body"] = err["action"] + " not permitted for" + err["url"]
        return err

    @classmethod
    def get_api_error_message(cls, url, api, err):
        return "API Error" + cls.get_error_as_message(url, api, err)

    @classmethod
    def get_access_denied_message(cls, action, dbid, server):
        error_obj = cls.access_denied_err_obj(action, dbid, server)
        return "Access Denied" + cls.get_error_as_message(None, None, error_obj)

    @staticmethod
    def get_invalid_key_message(extra_message=""):
        message = "The Api KEY is Undefined " + extra_message
        return message

    @staticmethod
    def get_invalid_url_message(url, call):
        message = "Invalid argument to {}, {} is not a valid Terminus DB API endpoint".format(
            url, call
        )
        return message
